def run_python_script(script):
    # 创建一个字典来存储脚本执行的本地变量
    local_vars = {}

    try:
        # 执行脚本字符串
        exec(script, {}, local_vars)
        return local_vars['result']
    except Exception as e:
        return str(e)

if __name__ == '__main__':
    # 示例用法
    script = """
def add(a, b):
    return a + b

result = add(5, 3)
    """

    output = run_python_script(script)
    print(output)


import os
import re
import json
import copy
from pathlib import Path
from typing import AsyncGenerator, Set

import httpx
import requests
import tiktoken

from .base import BaseLLM
from ..utils.scripts import check_json
from ..tools import function_call_list
from ..plugins import PLUGINS, get_tools_result

def get_filtered_keys_from_object(obj: object, *keys: str) -> Set[str]:
    """
    Get filtered list of object variable names.
    :param keys: List of keys to include. If the first key is "not", the remaining keys will be removed from the class keys.
    :return: List of class keys.
    """
    class_keys = obj.__dict__.keys()
    if not keys:
        return set(class_keys)

    # Remove the passed keys from the class keys.
    if keys[0] == "not":
        return {key for key in class_keys if key not in keys[1:]}
    # Check if all passed keys are valid
    if invalid_keys := set(keys) - class_keys:
        raise ValueError(
            f"Invalid keys: {invalid_keys}",
        )
    # Only return specified keys that are in class_keys
    return {key for key in keys if key in class_keys}

class chatgpt(BaseLLM):
    """
    Official ChatGPT API
    """

    def __init__(
        self,
        api_key: str,
        engine: str = os.environ.get("GPT_ENGINE") or "gpt-3.5-turbo",
        api_url: str = (os.environ.get("API_URL") or "https://api.openai.com/v1/chat/completions"),
        system_prompt: str = "You are ChatGPT, a large language model trained by OpenAI. Respond conversationally",
        proxy: str = None,
        timeout: float = 600,
        max_tokens: int = None,
        temperature: float = 0.5,
        top_p: float = 1.0,
        presence_penalty: float = 0.0,
        frequency_penalty: float = 0.0,
        reply_count: int = 1,
        truncate_limit: int = None,
        use_plugins: bool = True,
    ) -> None:
        """
        Initialize Chatbot with API key (from https://platform.openai.com/account/api-keys)
        """
        super().__init__(api_key, engine, api_url, system_prompt, proxy, timeout, max_tokens, temperature, top_p, presence_penalty, frequency_penalty, reply_count, truncate_limit, use_plugins=use_plugins)
        self.conversation: dict[str, list[dict]] = {
            "default": [
                {
                    "role": "system",
                    "content": system_prompt,
                },
            ],
        }
        self.function_calls_counter = {}
        self.function_call_max_loop = 3

        if self.get_token_count("default") > self.max_tokens:
            raise Exception("System prompt is too long")

    def add_to_conversation(
        self,
        message: list,
        role: str,
        convo_id: str = "default",
        function_name: str = "",
        total_tokens: int = 0,
    ) -> None:
        """
        Add a message to the conversation
        """
        if convo_id not in self.conversation:
            self.reset(convo_id=convo_id)
        if function_name == "" and message and message != None:
            self.conversation[convo_id].append({"role": role, "content": message})
        elif function_name != "" and message and message != None:
            self.conversation[convo_id].append({"role": role, "name": function_name, "content": message})
        else:
            print('\033[31m')
            print("error: add_to_conversation message is None or empty")
            print("role", role, "function_name", function_name, "message", message)
            print('\033[0m')
        if total_tokens:
            self.tokens_usage[convo_id] += total_tokens

    def __truncate_conversation(self, convo_id: str = "default") -> None:
        """
        Truncate the conversation
        """
        while True:
            if (
                self.get_token_count(convo_id) > self.truncate_limit
                and len(self.conversation[convo_id]) > 1
            ):
                # Don't remove the first message
                mess = self.conversation[convo_id].pop(1)
                print("Truncate message:", mess)
            else:
                break

    def truncate_conversation(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        model: str = None,
        pass_history: bool = True,
        **kwargs,
    ) -> None:
        """
        Truncate the conversation
        """
        while True:
            json_post = self.get_post_body(prompt, role, convo_id, model, pass_history, **kwargs)
            url = self.api_url.chat_url
            # if "gpt-4" in self.engine or "claude" in self.engine or (CUSTOM_MODELS and self.engine in CUSTOM_MODELS):
            message_token = {
                "total": self.get_token_count(convo_id),
            }
            # else:
            #     message_token = self.get_message_token(url, json_post)
            print("message_token", message_token, "truncate_limit", self.truncate_limit)
            if (
                message_token["total"] > self.truncate_limit
                and len(self.conversation[convo_id]) > 1
            ):
                # Don't remove the first message
                mess = self.conversation[convo_id].pop(1)
                print("Truncate message:", mess)
            else:
                break
        return json_post, message_token

    def extract_values(self, obj):
        if isinstance(obj, dict):
            for value in obj.values():
                yield from self.extract_values(value)
        elif isinstance(obj, list):
            for item in obj:
                yield from self.extract_values(item)
        else:
            yield obj
    # def clear_function_call(self, convo_id: str = "default"):
    #     self.conversation[convo_id] = [item for item in self.conversation[convo_id] if '@Trash@' not in item['content']]
    #     function_call_items = [item for item in self.conversation[convo_id] if 'function' in item['role']]
    #     function_call_num = len(function_call_items)
    #     if function_call_num > 50:
    #         for i in range(function_call_num - 25):
    #             self.conversation[convo_id].remove(function_call_items[i])

    # https://github.com/openai/openai-cookbook/blob/main/examples/How_to_count_tokens_with_tiktoken.ipynb
    def get_token_count(self, convo_id: str = "default") -> int:
        """
        Get token count
        """
        encoding = tiktoken.get_encoding("cl100k_base")

        num_tokens = 0
        for message in self.conversation[convo_id]:
            # every message follows <im_start>{role/name}\n{content}<im_end>\n
            num_tokens += 5
            for key, value in message.items():
                values = list(self.extract_values(value))
                if "image_url" in values:
                    continue
                if values:
                    for value in values:
                        # print("value", value)
                        num_tokens += len(encoding.encode(value))
                if key == "name":  # if there's a name, the role is omitted
                    num_tokens += 5  # role is always required and always 1 token
        num_tokens += 5  # every reply is primed with <im_start>assistant
        return num_tokens

    def get_message_token(self, url, json_post):
        json_post["max_tokens"] = 17000
        headers = {"Authorization": f"Bearer {os.environ.get('API', None)}"}
        response = requests.Session().post(
            url,
            headers=headers,
            json=json_post,
            timeout=None,
        )
        if response.status_code != 200:
            print(response.text)
            json_response = json.loads(response.text)
            string = json_response["error"]["message"]
            # print(json_response)
            try:
                string = re.findall(r"\((.*?)\)", string)[0]
            except:
                if "You exceeded your current quota" in json_response:
                    raise Exception("当前账号余额不足！")
            numbers = re.findall(r"\d+\.?\d*", string)
            numbers = [int(i) for i in numbers]
            if len(numbers) == 2:
                return {
                    "messages": numbers[0],
                    "total": numbers[0],
                }
            elif len(numbers) == 3:
                return {
                    "messages": numbers[0],
                    "functions": numbers[1],
                    "total": numbers[0] + numbers[1],
                }
            else:
                raise Exception(json_post, json_response)
        # print("response.text", response.text)
        return {
            "messages": 0,
            "total": 0,
        }


    def get_post_body(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        model: str = None,
        pass_history: bool = True,
        **kwargs,
    ):
        json_post_body = {
            "model": model or self.engine,
            "messages": self.conversation[convo_id] if pass_history else [{"role": "system","content": self.system_prompt},{"role": role, "content": prompt}],
            "max_tokens": 5000,
            "stream": True,
        }
        body = {
            # kwargs
            "temperature": kwargs.get("temperature", self.temperature),
            "top_p": kwargs.get("top_p", self.top_p),
            "presence_penalty": kwargs.get(
                "presence_penalty",
                self.presence_penalty,
            ),
            "frequency_penalty": kwargs.get(
                "frequency_penalty",
                self.frequency_penalty,
            ),
            "n": kwargs.get("n", self.reply_count),
            "user": role,
        }
        json_post_body.update(copy.deepcopy(body))
        if all(value == False for value in self.plugins[convo_id].values()) or self.use_plugins == False:
            return json_post_body
        json_post_body.update(copy.deepcopy(function_call_list["base"]))
        for item in self.plugins[convo_id].keys():
            try:
                if self.plugins[convo_id][item]:
                    json_post_body["functions"].append(function_call_list[item])
            except:
                pass

        return json_post_body

    def get_max_tokens(self, convo_id: str) -> int:
        """
        Get max tokens
        """
        # print("self.max_tokens, self.get_token_count(convo_id)", self.max_tokens, self.get_token_count(convo_id))
        return self.max_tokens - self.get_token_count(convo_id)

    def ask_stream(
        self,
        prompt: list,
        role: str = "user",
        convo_id: str = "default",
        model: str = None,
        pass_history: bool = True,
        function_name: str = "",
        total_tokens: int = 0,
        **kwargs,
    ):
        """
        Ask a question
        """
        # Make conversation if it doesn't exist
        if convo_id not in self.conversation or pass_history == False:
            self.reset(convo_id=convo_id, system_prompt=self.system_prompt)
        self.add_to_conversation(prompt, role, convo_id=convo_id, function_name=function_name, total_tokens=total_tokens)
        json_post, message_token = self.truncate_conversation(prompt, role, convo_id, model, pass_history, **kwargs)
        # print(self.conversation[convo_id])
        model_max_tokens = kwargs.get("max_tokens", self.max_tokens)
        print("model_max_tokens", model_max_tokens)
        json_post["max_tokens"] = model_max_tokens
        print("api_url", self.api_url.chat_url)
        for _ in range(2):
            print(json.dumps(json_post, indent=4, ensure_ascii=False))
            try:
                response = self.session.post(
                    self.api_url.chat_url,
                    headers={"Authorization": f"Bearer {kwargs.get('api_key', self.api_key)}"},
                    json=json_post,
                    timeout=kwargs.get("timeout", self.timeout),
                    stream=True,
                )
            except ConnectionError:
                print("连接错误，请检查服务器状态或网络连接。")
                return
            except requests.exceptions.ReadTimeout:
                print("请求超时，请检查网络连接或增加超时时间。{e}")
                return
            except Exception as e:
                print(f"发生了未预料的错误: {e}")
                return
            # print("response.text", response.text)
            if response.status_code == 400:
                print("response.text", response.text)
                if "invalid_request_error" in response.text:
                    for index, mess in enumerate(json_post["messages"]):
                        if type(mess["content"]) == list:
                            json_post["messages"][index] = {
                                "role": mess["role"],
                                "content": mess["content"][0]["text"]
                            }
                elif "'function' is not an allowed role" in response.text:
                    if json_post["messages"][-1]["role"] == "function":
                        mess = json_post["messages"][-1]
                        json_post["messages"][-1] = {
                            "role": "assistant",
                            "name": mess["name"],
                            "content": mess["content"]
                        }
                else:
                    if "function_call" in json_post:
                        del json_post["function_call"]
                    if "functions" in json_post:
                        del json_post["functions"]
                continue
            if response.status_code == 200:
                break
        if response.status_code != 200:
            raise Exception(f"{response.status_code} {response.reason} {response.text}")
        response_role: str = None
        full_response: str = ""
        function_full_response: str = ""
        function_call_name: str = ""
        need_function_call: bool = False
        total_tokens = 0
        for line in response.iter_lines():
            if not line or line.decode("utf-8").startswith(':'):
                continue
            # print("line.decode('utf-8')", line.decode("utf-8"))
            if line.decode("utf-8").startswith('data:'):
                line = line.decode("utf-8")[6:]
            else:
                # print("line", line.decode("utf-8"))
                full_response = json.loads(line.decode("utf-8"))["choices"][0]["message"]["content"]
                yield full_response
                break
            if line == "[DONE]":
                break
            resp: dict = json.loads(line)
            # print("resp", resp)
            usage = resp.get("usage")
            if usage:
                total_tokens = usage.get("total_tokens") or 0
                print("\n\rtotal_tokens", total_tokens)
            choices = resp.get("choices")
            if not choices:
                continue
            delta = choices[0].get("delta")
            if not delta:
                continue
            if "role" in delta and response_role == None:
                response_role = delta["role"]
            if "content" in delta and delta["content"]:
                need_function_call = False
                content = delta["content"]
                full_response += content
                yield content
            if "function_call" in delta:
                # print(delta["function_call"], end="")
                need_function_call = True
                function_call_content = delta["function_call"]["arguments"]
                if "name" in delta["function_call"]:
                    function_call_name = delta["function_call"]["name"]
                function_full_response += function_call_content
                if function_full_response.count("\\n") > 2 or "}" in function_full_response:
                    break
        if need_function_call:
            function_full_response = check_json(function_full_response)
            print("function_full_response", function_full_response)
            function_response = ""
            # print(self.function_calls_counter)
            if not self.function_calls_counter.get(function_call_name):
                self.function_calls_counter[function_call_name] = 1
            else:
                self.function_calls_counter[function_call_name] += 1
            if self.function_calls_counter[function_call_name] <= self.function_call_max_loop:
                function_call_max_tokens = self.truncate_limit - message_token["total"] - 1000
                if function_call_max_tokens <= 0:
                    function_call_max_tokens = int(self.truncate_limit / 2)
                print("\033[32m function_call", function_call_name, "max token:", function_call_max_tokens, "\033[0m")
                function_response = yield from get_tools_result(function_call_name, function_full_response, function_call_max_tokens, self.engine, chatgpt, self.api_key, self.api_url, use_plugins=False, model=model, add_message=self.add_to_conversation, convo_id=convo_id)
            else:
                function_response = "无法找到相关信息，停止使用 tools"
            response_role = "function"
            # print(self.conversation[convo_id][-1])
            if self.conversation[convo_id][-1]["role"] == "function" and self.conversation[convo_id][-1]["name"] == "get_search_results":
                mess = self.conversation[convo_id].pop(-1)
                # print("Truncate message:", mess)
            yield from self.ask_stream(function_response, response_role, convo_id=convo_id, function_name=function_call_name, total_tokens=total_tokens, model=model)
        else:
            if self.conversation[convo_id][-1]["role"] == "function" and self.conversation[convo_id][-1]["name"] == "get_search_results":
                mess = self.conversation[convo_id].pop(-1)
            self.add_to_conversation(full_response, response_role, convo_id=convo_id, total_tokens=total_tokens)
            self.function_calls_counter = {}
            # self.clear_function_call(convo_id=convo_id)
            # self.encode_web_text_list = []
            # total_tokens = self.get_token_count(convo_id)

    async def ask_stream_async(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        model: str = None,
        pass_history: bool = True,
        **kwargs,
    ) -> AsyncGenerator[str, None]:
        """
        Ask a question
        """
        # Make conversation if it doesn't exist
        if convo_id not in self.conversation or pass_history == False:
            self.reset(convo_id=convo_id, system_prompt=self.system_prompt)
        self.add_to_conversation(prompt, "user", convo_id=convo_id)
        self.__truncate_conversation(convo_id=convo_id)
        if self.engine == "gpt-4-1106-preview" or "gpt-4-0125-preview" in self.engine or "gpt-4-turbo" in self.engine:
            model_max_tokens = kwargs.get("max_tokens", self.max_tokens)
        else:
            model_max_tokens = min(self.get_max_tokens(convo_id=convo_id) - 500, kwargs.get("max_tokens", self.max_tokens))
        # Get response
        async with self.aclient.stream(
            "post",
            self.api_url.chat_url,
            headers={"Authorization": f"Bearer {kwargs.get('api_key', self.api_key)}"},
            json={
                "model": model or self.engine,
                "messages": self.conversation[convo_id] if pass_history else [{"role": "system","content": self.system_prompt},{"role": role, "content": prompt}],
                "stream": True,
                # kwargs
                "temperature": kwargs.get("temperature", self.temperature),
                "top_p": kwargs.get("top_p", self.top_p),
                "presence_penalty": kwargs.get(
                    "presence_penalty",
                    self.presence_penalty,
                ),
                "frequency_penalty": kwargs.get(
                    "frequency_penalty",
                    self.frequency_penalty,
                ),
                "n": kwargs.get("n", self.reply_count),
                "user": role,
                "max_tokens": model_max_tokens,
                # "max_tokens": min(
                #     self.get_max_tokens(convo_id=convo_id),
                #     kwargs.get("max_tokens", self.max_tokens),
                # ),
            },
            timeout=kwargs.get("timeout", self.timeout),
        ) as response:
            if response.status_code != 200:
                await response.aread()
                raise Exception(f"{response.status_code} {response.reason_phrase} {response.text}")
            response_role: str = ""
            full_response: str = ""
            async for line in response.aiter_lines():
                line = line.strip()
                if not line:
                    continue
                # Remove "data: "
                line = line[6:]
                if line == "[DONE]":
                    break
                resp: dict = json.loads(line)
                if "error" in resp:
                    raise Exception(f"{resp['error']}")
                choices = resp.get("choices")
                if not choices:
                    continue
                delta: dict[str, str] = choices[0].get("delta")
                if not delta:
                    continue
                if "role" in delta:
                    response_role = delta["role"]
                if "content" in delta:
                    content: str = delta["content"]
                    full_response += content
                    yield content
        self.add_to_conversation(full_response, response_role, convo_id=convo_id)
        print("total tokens:", self.get_token_count(convo_id))

    async def ask_async(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        model: str = None,
        pass_history: bool = True,
        **kwargs,
    ) -> str:
        """
        Non-streaming ask
        """
        response = self.ask_stream_async(
            prompt=prompt,
            role=role,
            convo_id=convo_id,
            **kwargs,
        )
        full_response: str = "".join([r async for r in response])
        return full_response

    def rollback(self, n: int = 1, convo_id: str = "default") -> None:
        """
        Rollback the conversation
        """
        for _ in range(n):
            self.conversation[convo_id].pop()

    def reset(self, convo_id: str = "default", system_prompt: str = None) -> None:
        """
        Reset the conversation
        """
        self.conversation[convo_id] = [
            {"role": "system", "content": system_prompt or self.system_prompt},
        ]
        if convo_id not in self.plugins:
            self.plugins[convo_id] = copy.deepcopy(PLUGINS)

    def save(self, file: str, *keys: str) -> None:
        """
        Save the Chatbot configuration to a JSON file
        """
        with open(file, "w", encoding="utf-8") as f:
            data = {
                key: self.__dict__[key]
                for key in get_filtered_keys_from_object(self, *keys)
            }
            # saves session.proxies dict as session
            # leave this here for compatibility
            data["session"] = data["proxy"]
            del data["aclient"]
            json.dump(
                data,
                f,
                indent=2,
            )

    def load(self, file: Path, *keys_: str) -> None:
        """
        Load the Chatbot configuration from a JSON file
        """
        with open(file, encoding="utf-8") as f:
            # load json, if session is in keys, load proxies
            loaded_config = json.load(f)
            keys = get_filtered_keys_from_object(self, *keys_)

            if (
                "session" in keys
                and loaded_config["session"]
                or "proxy" in keys
                and loaded_config["proxy"]
            ):
                self.proxy = loaded_config.get("session", loaded_config["proxy"])
                self.session = httpx.Client(
                    follow_redirects=True,
                    proxies=self.proxy,
                    timeout=self.timeout,
                    cookies=self.session.cookies,
                    headers=self.session.headers,
                )
                self.aclient = httpx.AsyncClient(
                    follow_redirects=True,
                    proxies=self.proxy,
                    timeout=self.timeout,
                    cookies=self.session.cookies,
                    headers=self.session.headers,
                )
            if "session" in keys:
                keys.remove("session")
            if "aclient" in keys:
                keys.remove("aclient")
            self.__dict__.update({key: loaded_config[key] for key in keys})


import os
from rich.console import Console
from rich.markdown import Markdown
import json
import requests

def query_ollama(prompt, model):
    # 设置请求的URL和数据
    url = 'http://localhost:11434/api/generate'
    data = {
        "model": model,
        "prompt": prompt,
        "stream": True,
    }

    response = requests.Session().post(
        url,
        json=data,
        stream=True,
    )
    full_response: str = ""
    for line in response.iter_lines():
        if not line or line.decode("utf-8")[:6] == "event:" or line.decode("utf-8") == "data: {}":
            continue
        line = line.decode("utf-8")
        # print(line)
        resp: dict = json.loads(line)
        content = resp.get("response")
        if not content:
            continue
        full_response += content
        yield content

if __name__ == "__main__":
    console = Console()
    # model = 'llama2'
    # model = 'mistral'
    # model = 'llama3:8b'
    model = 'phi3:medium'
    # model = 'qwen:14b'
    # model = 'wizardlm2:7b'
    # model = 'codeqwen:7b-chat'
    # model = 'phi'

    # 查询答案
    prompt = r'''

class MotionAGFormer(nn.Module):
    """
    MotionAGFormer, the main class of our model.
    """

    def __init__(self, n_layers, dim_in, dim_feat, dim_rep=512, dim_out=3, mlp_ratio=4, act_layer=nn.GELU, attn_drop=0.,
                 drop=0., drop_path=0., use_layer_scale=True, layer_scale_init_value=1e-5, use_adaptive_fusion=True,
                 num_heads=4, qkv_bias=False, qkv_scale=None, hierarchical=False, num_joints=17,
                 use_temporal_similarity=True, temporal_connection_len=1, use_tcn=False, graph_only=False,
                 neighbour_num=4, n_frames=243, dir_num=4, dir_act='tanh', edge_num=2, edge_act='tanh', rel0=True, h36=True):
        """
        :param n_layers: Number of layers.
        :param dim_in: Input dimension.
        :param dim_feat: Feature dimension.
        :param dim_rep: Motion representation dimension
        :param dim_out: output dimension. For 3D pose lifting it is set to 3
        :param mlp_ratio: MLP ratio.
        :param act_layer: Activation layer.
        :param drop: Dropout rate.
        :param drop_path: Stochastic drop probability.
        :param use_layer_scale: Whether to use layer scaling or not.
        :param layer_scale_init_value: Layer scale init value in case of using layer scaling.
        :param use_adaptive_fusion: Whether to use adaptive fusion or not.
        :param num_heads: Number of attention heads in attention branch
        :param qkv_bias: Whether to include bias in the linear layers that create query, key, and value or not.
        :param qkv_scale: scale factor to multiply after outer product of query and key. If None, it's set to
                          1 / sqrt(dim_feature // num_heads)
        :param hierarchical: Whether to use hierarchical structure or not.
        :param num_joints: Number of joints.
        :param use_temporal_similarity: If true, for temporal GCN uses top-k similarity between nodes
        :param temporal_connection_len: Connects joint to itself within next `temporal_connection_len` frames
        :param use_tcn: If true, uses MS-TCN for temporal part of the graph branch.
        :param graph_only: Uses GCN instead of GraphFormer in the graph branch.
        :param neighbour_num: Number of neighbors for temporal GCN similarity.
        :param n_frames: Number of frames. Default is 243
        """
        super().__init__()
        self.h36 = h36
        if h36:
            self.root_id = 0
            self.limbs_id = [[0, 1], [1, 2], [2, 3],
                    [0, 4], [4, 5], [5, 6],
                    [0, 7], [7, 8], [8, 9], [9, 10],
                    [8, 11], [11, 12], [12, 13],
                    [8, 14], [14, 15], [15, 16]
                    ]
        else:
            self.root_id = 14
            self.limbs_id = [[14, 8], [8, 9], [9, 10],
                             [14, 11], [11, 12], [12, 13],
                             [14, 15], [15, 1], [1, 16], [16, 0],
                             [1, 5], [5, 6], [6, 7],
                             [1, 2], [2, 3], [3, 4]]

        self.joints_embed = nn.Linear(dim_in, dim_feat)
        # self.inp_embed = nn.Linear(dim_feat * 2, dim_feat)
        self.pos_embed = nn.Parameter(torch.zeros(1, num_joints, dim_feat))
        # self.pos_embed2 = nn.Parameter(torch.zeros(1, num_joints, dim_feat))
        self.norm = nn.LayerNorm(dim_feat)

        self.layers = create_layers(dim=dim_feat,
                                    n_layers=n_layers,
                                    mlp_ratio=mlp_ratio,
                                    act_layer=act_layer,
                                    attn_drop=attn_drop,
                                    drop_rate=drop,
                                    drop_path_rate=drop_path,
                                    num_heads=num_heads,
                                    use_layer_scale=use_layer_scale,
                                    qkv_bias=qkv_bias,
                                    qkv_scale=qkv_scale,
                                    layer_scale_init_value=layer_scale_init_value,
                                    use_adaptive_fusion=use_adaptive_fusion,
                                    hierarchical=hierarchical,
                                    use_temporal_similarity=use_temporal_similarity,
                                    temporal_connection_len=temporal_connection_len,
                                    use_tcn=use_tcn,
                                    graph_only=graph_only,
                                    neighbour_num=neighbour_num,
                                    n_frames=n_frames)
        # self.layers_edge = create_layers(dim=dim_feat,
        #                             n_layers=n_layers//2,
        #                             mlp_ratio=mlp_ratio,
        #                             act_layer=act_layer,
        #                             attn_drop=attn_drop,
        #                             drop_rate=drop,
        #                             drop_path_rate=drop_path,
        #                             num_heads=num_heads,
        #                             use_layer_scale=use_layer_scale,
        #                             qkv_bias=qkv_bias,
        #                             qkv_scale=qkv_scale,
        #                             layer_scale_init_value=layer_scale_init_value,
        #                             use_adaptive_fusion=use_adaptive_fusion,
        #                             hierarchical=hierarchical,
        #                             use_temporal_similarity=use_temporal_similarity,
        #                             temporal_connection_len=temporal_connection_len,
        #                             use_tcn=use_tcn,
        #                             graph_only=graph_only,
        #                             neighbour_num=neighbour_num,
        #                             n_frames=n_frames)

        # self.rep_logit_dir = nn.Sequential(OrderedDict([
        #     ('fc', nn.Linear(dim_feat, dim_rep)),
        #     ('act', nn.Tanh())
        # ]))
        # self.rep_logit_edge = nn.Sequential(OrderedDict([
        #     ('fc', nn.Linear(dim_feat, dim_rep)),
        #     ('act', nn.Tanh())
        # ]))

        # self.head = nn.Linear(dim_rep, dim_out)
        self.rel0 = rel0
        self.dir_act = self.get_act(dir_act)
        self.edge_act = self.get_act(edge_act)
        if self.rel0:
            self.dir_head = MLP_zsr(dim_feat, dim_rep, 3, dir_num, drop_out=0.0, act=self.dir_act)
            self.edge_head = MLP_zsr(dim_feat, dim_rep, 1, edge_num, drop_out=0.0, act=self.edge_act)
        else:
            self.pose_head = PredHead(dim_feat, dim_rep, 3, 0, dir_num, 0, act=self.dir_act)
            self.dir_head = PredHead(dim_feat, dim_rep, 3, 0, dir_num, 0, act=self.dir_act)
            self.edge_head = PredHead(dim_feat, dim_rep, 3, 0, edge_num, 0, act=self.edge_act)

    def get_act(self, act):
        if act == 'tanh':
            return nn.Tanh()
        elif act == 'relu':
            return nn.ReLU()

    def get_limbs(self, x):
        B, T, J, C = x.shape
        limbs = x[:, :, self.limbs_id, :]
        limbs = limbs[:, :, :, 1, :] - limbs[:, :, :, 0, :]  # (B, T, J-1, C)
        res = torch.zeros((B, T, J, C), device=x.device)
        res[:,:,1:,:] = limbs[:,:,:,:]
        return res
    def get_dir(self, x):
        limbs = x[:, :, self.limbs_id, :]
        limbs = limbs[:, :, :, 1, :] - limbs[:, :, :, 0, :]     # (b, f, 16, 3)
        limbs = F.normalize(limbs, dim=-1)
        return limbs

    def get_edge(self, x):
        limbs = x[:, :, self.limbs_id, :]
        limbs = limbs[:, :, :, 1, :] - limbs[:, :, :, 0, :]     # (b, f, 16, 3)
        limbs_len = torch.norm(limbs, dim=-1, keepdim=True)
        return limbs_len

    def dir_forward(self, x):
        x = self.dir_head(x)        # (bs, f, 17, 3)
        x = self.get_dir(x)         # (bs, f, 16, 3)
        return x

    def edge_forward(self, x):
        x = self.edge_head(x)       # (b, f, 17, 3)
        x = self.get_edge(x)        # (b, f, 16, 1)
        return x

    def combine_dir_edge(self, x_dir, x_edge):
        shape = list(x_dir.shape)
        shape[-2] = 17
        pose = torch.zeros(shape, device=x_dir.device)
        for i in range(16):
            k0, k1 = self.limbs_id[i]
            pose[:,:,k1] = pose[:,:,k0] + x_dir[:,:, i] * (x_edge[:,:, i].clamp(min=1e-3))
        return pose
    # def combine_dir_edge_detach(self, x_dir, x_edge):
    #     assert x_dir.shape[-1] == 3 and x_edge.shape[-1] == 1
    #     # detach
    #     dir_length = torch.norm(x_dir, dim=-1, keepdim=True).clamp(min=1e-9).detach()
    #     x_dir = x_dir / dir_length
    #     pose = x_dir * x_edge
    #     pose[:,:,0] = 0
    #     return pose, x_dir, x_edge

    def forward(self, x, return_rep=False):
        """
        :param x: tensor with shape [B, T, J, C] (T=243, J=17, C=3)
        :param return_rep: Returns motion representation feature volume (In case of using this as backbone)
        """
        eps = 1e-7
        x = self.joints_embed(x)
        x = x + self.pos_embed
        # x_l = self.get_limbs(x)
        # x = self.inp_embed(torch.cat([x, x_l], dim=-1)) + self.pos_embed2     # 再添加一个pos_embed
        for layer in self.layers:
            x = layer(x)
        x = self.norm(x)
        if self.rel0:
            x_dir = self.dir_head(x)
            x_edge = self.edge_head(x)
            x_dir = F.normalize(x_dir, dim=-1)
            x_dir[...,self.root_id,:] = 0
            pose = x_dir * (x_edge.clamp(min=1e-3))
            return pose, x_dir, x_edge
        else:
            # pose_fusion = self.pose_fusion(x).softmax(dim=-1)   # (B, T, J, 2)
            pose_0 = self.pose_head(x)
            x_dir_0 = self.get_dir(pose_0)
            x_edge_0 = self.get_edge(pose_0)
            # pose = pose_0
            # x_dir = x_dir_0
            # x_edge = x_edge_0
            x_dir_1 = self.dir_forward(x)     # (bs, f, 16, 3)
            x_edge_1 = self.edge_forward(x)   # (bs, f, 16, 1)
            pose_1 = self.combine_dir_edge(x_dir_1, x_edge_1)
            # pose = pose_1
            # x_dir = x_dir_1
            # x_edge = x_edge_1
            x_dir = (x_dir_0 + x_dir_1) / 2
            x_edge = (x_edge_0 + x_edge_1) / 2
            pose = (pose_0 + pose_1)/2
            return pose, x_dir, x_edge

        # for layer in self.layers:
        #     x = layer(x)

        # x = self.norm(x)
        # x_dir = self.rep_logit_dir(x)   # (bs, f, 17, 512)
        # if return_rep:
        #     return x

        # x = self.head(x)
        # x_dir = self.dir_proj(x.transpose(-1, -2)).transpose(-1,-2)
        # x_dir = self.dir_head(x_dir)    # (bs, f, 16, 3)
        # x_scale = self.scale_proj(x.transpose(-1, -2)).transpose(-1, -2)
        # x_scale = self.scale_head(x_scale)  # (bs, f, 16, 1)
        # pose, uni_dir = combine(x_dir, x_scale)
        # return x
        # return uni_dir, x_scale, pose
分析一下这个代码

'''
    answer = ""
    for result in query_ollama(prompt, model):
        os.system("clear")
        answer += result
        md = Markdown(answer)
        console.print(md, no_wrap=False)



import os
from googleapiclient.discovery import build
from dotenv import load_dotenv
load_dotenv()

search_engine_id = os.environ.get('GOOGLE_CSE_ID', None)
api_key = os.environ.get('GOOGLE_API_KEY', None)

def google_search(query, api_key, search_engine_id):
    service = build("customsearch", "v1", developerKey=api_key)
    res = service.cse().list(q=query, cx=search_engine_id).execute()
    link_list = [item['link'] for item in res['items']]
    return link_list

# 执行搜索
query = "Python programming"
results = google_search(query, api_key, search_engine_id)
print(results)

# # 处理搜索结果
# for result in results:
#     print(result['title'])
#     print(result['link'])
#     print(result['snippet'])
#     print("---")